﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Bulldozer
{
    public class DrawningBulldozer
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBulldozer? EntityBulldozer { get; private set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки бульдозера
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки бульдозера
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки бульдозера
        /// </summary>
        private readonly int bulldozerWidth = 170;
        /// <summary>
        /// Высота прорисовки бульдозера
        /// </summary>
        private readonly int bulldozerHeight = 85;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bulldozerColor">Основной цвет</param>
        /// <param name="cabinColor">Дополнительный цвет</param>
        /// <param name="covshColor">Цвет для ковша</param>
        /// <param name="hasMoldboardfront">Признак наличия переднего ковша</param>
        /// <param name="hasRipper">Признак наличия заднего ковша</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена,
        public bool Init(int speed, double weight, Color bulldozerColor, Color cabinColor, Color covshColor, bool hasMoldboardfront, bool hasRipper, int width, int height)
        {
            if (width < _pictureWidth || height < _pictureHeight)
            {
                return false;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityBulldozer = new EntityBulldozer();
            EntityBulldozer.Init(speed, weight, bulldozerColor, cabinColor, covshColor, hasMoldboardfront, hasRipper);
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x < 0 || y < 0 || x + bulldozerWidth > _pictureWidth || y + bulldozerHeight > _pictureHeight)
            {
                x = 10;
                y = 10;
            }
            _startPosX = x;
            _startPosY = y;
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionTypeBulldozer direction)
        {
            if (EntityBulldozer == null)

            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionTypeBulldozer.Left:
                    if (_startPosX - EntityBulldozer.Step > 0)
                    {
                        _startPosX -= (int)EntityBulldozer.Step;
                    }
                    if (_startPosX - EntityBulldozer.Step < 0)
                    {
                        _startPosX -= _startPosX - (int)EntityBulldozer.Step;
                    }
                    break;
                //вверх
                case DirectionTypeBulldozer.Up:
                    if (_startPosY - EntityBulldozer.Step > 0)
                    {
                        _startPosY -= (int)EntityBulldozer.Step;
                    }
                    else if (_startPosY - EntityBulldozer.Step < 0)
                    {
                        _startPosY -= _startPosY - (int)EntityBulldozer.Step;
                    }
                    break;
                // вправо
                case DirectionTypeBulldozer.Right:
                    if (_startPosX + EntityBulldozer.Step + bulldozerWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityBulldozer.Step;
                    }
                    else if (_startPosX + EntityBulldozer.Step + bulldozerWidth > _pictureWidth)
                    {
                        _startPosX += _pictureWidth - _startPosX - bulldozerWidth;
                    }
                    break;
                //вниз
                case DirectionTypeBulldozer.Down:
                    if (_startPosY + EntityBulldozer.Step + bulldozerHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityBulldozer.Step;
                    }
                    else if (_startPosY + EntityBulldozer.Step + bulldozerHeight > _pictureHeight)
                    {
                        _startPosY += _pictureHeight - _startPosY - bulldozerHeight;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityBulldozer == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityBulldozer.CabinColor);

            // Тело трактора
            Brush bulldozerColor = new SolidBrush(EntityBulldozer.BulldozerColor);
            g.FillRectangle(bulldozerColor, _startPosX + 25, _startPosY + 20, 110, 30);
            g.FillRectangle(bulldozerColor, _startPosX + 60, _startPosY, 10, 30);

            int x = _startPosX + 30; // начальная позиция X
            int y = _startPosY; // начальная позиция Y
            int width = 110; // ширина прямоугольника
            int height = 30; // высота прямоугольника
            int radius = 20; // радиус закругления углов

            // Рисуем закругленный прямоугольник
            g.DrawArc(pen, x - 5, y + 50, radius * 2, radius * 2, 180, 90); // верхний левый угол
            g.DrawLine(pen, x + radius - 5, y + 50, x + width - radius - 5, y + 50); // верхняя горизонталь
            g.DrawArc(pen, x + width - radius * 2 - 5, y + 50, radius * 2, radius * 2, 270, 90); // верхний правый угол
            g.DrawArc(pen, x + width - radius * 2 - 5, y + height - radius * 2 + 50, radius * 2, radius * 2, 0, 90); // нижний правый угол
            g.DrawLine(pen, x + width - radius - 5, y + height + 50, x + radius - 5, y + height + 50); // нижняя горизонталь
            g.DrawArc(pen, x - 5, y + height - radius * 2 + 50, radius * 2, radius * 2, 90, 90); // нижний левый угол

            int wheelRadius = 15;

            // Рисуем колеса трактора
            g.DrawEllipse(pen, _startPosX + 30, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.FillEllipse(additionalBrush, _startPosX + 30, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.DrawEllipse(pen, _startPosX + 65, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.FillEllipse(additionalBrush, _startPosX + 65, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.DrawEllipse(pen, _startPosX + 100, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);
            g.FillEllipse(additionalBrush, _startPosX + 100, _startPosY + 50, wheelRadius * 2, wheelRadius * 2);

            // Кабина
            Brush cabinColor = new SolidBrush(EntityBulldozer.CabinColor);
            g.FillRectangle(cabinColor, _startPosX + 105, _startPosY, 30, 20);

            // Рисуем ковш спереди
            if (EntityBulldozer.HasMoldboardfront)
            {
                Point[] trianglePoints = new Point[]
                {
                new Point(_startPosX + 25, _startPosY + 30),
                new Point(_startPosX + 25, _startPosY + 80),
                new Point(_startPosX, _startPosY + 80),
                };
                g.DrawPolygon(pen, trianglePoints);
            }
            // Рисуем ковш сзади
            if (EntityBulldozer.HasRipper)
            {
                Point[] trianglePoints2 = new Point[]
                {
                    new Point(_startPosX + 130, _startPosY + 50),
                    new Point(_startPosX + 160, _startPosY + 50),
                    new Point(_startPosX + 160, _startPosY + 80)

                };
                g.DrawPolygon(pen, trianglePoints2);
            }
        }
    }
}
